/***************************************************

Copyright 2003 Ben Childs


This file is part of Physics 3D.

    Physics 3D is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Physics 3D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Physics 3D; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA




***************************************************/

package com.bchilds.Physics3D;


import javax.vecmath.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;
import java.text.*;

public abstract class Particle3DTracker extends JFrame
{
	
	private static ArrayList trackers = new ArrayList();
	protected Particle3D[] particles;
	
	protected static DecimalFormat formatter = new DecimalFormat("#.###E0");
	
	public Particle3DTracker()
	{
		super();
		setSize(200,50);
		
		Image img0 = getToolkit().getImage(ClassLoader.getSystemResource("physics3d.gif"));
		this.setIconImage(img0);
		
		trackers.add(this);
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				thisWindowClosing(e);
			}
		});
	
	}

	public boolean refersTo(Particle3D p)
	{
		if(particles == null)
			return false;
			
		for(int i = 0; i < particles.length; i++)
		{
			if(particles[i] == p)
				return true;
		}
		return false;
	}
	
	public abstract void updateInfo();
	
	public static void updateAll()
	{
		for(int i = 0; i < trackers.size(); i++)
		{
			((Particle3DTracker)trackers.get(i)).updateInfo();
		}
	}
	
	public void thisWindowClosing(WindowEvent e)
	{
		trackers.remove(this);
	}
	
	public static void removeReferring(Particle3D p)
	{
		for(int i = 0; i < trackers.size(); i++)
		{
			Particle3DTracker t = ((Particle3DTracker)trackers.get(i));
			if(t.refersTo(p))
			{
				t.setVisible(false);
				t.dispose();
				t.thisWindowClosing(null);
			}
		}
	}
	
	public static boolean printAllTrackers(Graphics2D g2d, Dimension printSize)
	{
	
		if(trackers.size() == 0)
			return false;
	
	
		//Print Title
		g2d.setFont(g2d.getFont().deriveFont(16f).deriveFont(Font.BOLD));
		String titleString = "Physics3D Data";
		Rectangle2D.Float r = (Rectangle2D.Float)g2d.getFont().getStringBounds(titleString, g2d.getFontRenderContext());
		
		g2d.drawString(titleString, (float)((printSize.getWidth() / 2) - (r.getWidth() / 2)), (float)r.getHeight());
		
		g2d.drawLine(0,(int)r.getHeight() + 10, (int)printSize.getWidth(), (int)r.getHeight() + 10);
	
	
		//Print Tracker Data
		g2d.setFont(g2d.getFont().deriveFont(12f).deriveFont(Font.PLAIN));
		Point currentPosition = new Point(0, (int)r.getHeight() + 20);
		
		for(int i = 0; i < trackers.size(); i++)
		{
			String current = ((Particle3DTracker)trackers.get(i)).toString();
			r = (Rectangle2D.Float)g2d.getFont().getStringBounds(current, g2d.getFontRenderContext());
			currentPosition.y += (int)r.getHeight();
			g2d.drawString(current, currentPosition.x, currentPosition.y);
			
		}
		
		return true;
	}
	
	
	
}