/***************************************************

Copyright 2003 Ben Childs


This file is part of Physics 3D.

    Physics 3D is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Physics 3D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Physics 3D; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA




***************************************************/

package com.bchilds.Physics3D;

//Java3D Imports
import javax.media.j3d.*;
import com.sun.j3d.utils.universe.*;
import javax.vecmath.*;
///////////////////////

//UI Imports
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.print.*;
//////////////////////////

public class Physics3DFrame extends javax.swing.JFrame implements Printable, MouseListener {


//Codewarrior RAD stuff....

	//Main Panels
	JPanel main3DPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	JPanel statusPanel = new JPanel();
	///////////////////////////
	
	//Main Toolbar Buttons
	JButton addParticleButton = new JButton();
	JButton removeButton = new JButton();
	JButton stepForwardButton = new JButton();
	JButton startButton = new JButton();
	JButton reverseButton = new JButton();
	JButton stepBackButton = new JButton();
	JButton stopButton = new JButton();
	JButton resetParticlesButton = new JButton();
	JButton resetViewButton = new JButton();
	////////////////////////////////
	
	//Menu's
	private JMenuBar mainMenu = new JMenuBar();
	private JMenu fileMenu = new JMenu();
	private JMenuItem fileMenuNew = new JMenuItem();
	private JMenuItem fileMenuSave = new JMenuItem();
	private JMenuItem fileMenuSaveAs = new JMenuItem();
	private JMenuItem fileMenuOpen = new JMenuItem();
	private JMenuItem fileMenuPrint = new JMenuItem();
	private JMenuItem fileMenuQuit = new JMenuItem();
	private JMenu jMenuEdit = new JMenu();
	private JMenuItem jMenuEditSettings = new JMenuItem();
	private JMenu jMenuTools = new JMenu();
	private JMenu jMenuToolsTrackers = new JMenu();
	private JMenuItem jMenuTrackersCreateTimeTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreatePositionTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateVelocityTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateMomentumTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateKineticEnergyTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateDistanceTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateOrientationTracker = new JMenuItem();
	private JMenuItem jMenuTrackersCreateAngularVelocityTracker = new JMenuItem();

	
	private JMenuItem jMenuToolsCreateRandomParticleGrid = new JMenuItem();
	
	
	
	private JMenu jMenuHelp = new JMenu();
	private JMenuItem jMenuHelpAbout = new JMenuItem();
	////////////////////////////////////////////////////
	
	//File Chooser for loading/saving
	private JFileChooser chooser;
	
	//Current Filename
	private File fileName = null;
	
	//Status Label
	JLabel statusLabel = new JLabel();
	///////////////

	//Main simulation thread
	Physics3DThread t = null;
	
	//Main Canvas3D
	private Canvas3D c;
	
	//OffScreen Canvas for Printing
	private OffScreenCanvas3D offScreenCanvas3D;

	public Physics3DFrame() {
	}

	public void initComponents() throws Exception {
		
		JSeparator fileMenuSeparator2 = new JSeparator();
		JSeparator fileMenuSeparator3 = new JSeparator();
		JSeparator fileMenuSeparator1 = new JSeparator();
		JSeparator fileMenuSeparator4 = new JSeparator();
		
		//Load Images for the Toolbar/Icon
		Image img0 = getToolkit().getImage(ClassLoader.getSystemResource("physics3d.gif"));
		Image img1 = getToolkit().getImage(ClassLoader.getSystemResource("add.gif"));
		Image img2 = getToolkit().getImage(ClassLoader.getSystemResource("remove.gif"));
		Image img3 = getToolkit().getImage(ClassLoader.getSystemResource("stepForward.gif"));
		Image img4 = getToolkit().getImage(ClassLoader.getSystemResource("start.gif"));
		Image img5 = getToolkit().getImage(ClassLoader.getSystemResource("reverse.gif"));
		Image img6 = getToolkit().getImage(ClassLoader.getSystemResource("stepBack.gif"));
		Image img7 = getToolkit().getImage(ClassLoader.getSystemResource("stop.gif"));
		Image img8 = getToolkit().getImage(ClassLoader.getSystemResource("rewind.gif"));
		Image img9 = getToolkit().getImage(ClassLoader.getSystemResource("view.gif"));
		////////////////////////////////////////////////////////////
		
		main3DPanel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		main3DPanel.setLayout(new BorderLayout(0, 0));
		main3DPanel.setVisible(true);
		
		buttonPanel.setBorder(new javax.swing.plaf.BorderUIResource.EtchedBorderUIResource(1, Color.white, new Color(142, 142, 142)));
		buttonPanel.setLayout(new BoxLayout(buttonPanel, 1));
		buttonPanel.setVisible(true);
		
		addParticleButton.setIcon(new ImageIcon(img1));
		addParticleButton.setMargin(new Insets(0, 0, 0, 0));
		addParticleButton.setToolTipText("Add Particle");
		addParticleButton.setVisible(true);
		
		removeButton.setIcon(new ImageIcon(img2));
		removeButton.setMargin(new Insets(0, 0, 0, 0));
		removeButton.setToolTipText("Remove Selected Object");
		removeButton.setVisible(true);
		
		stepForwardButton.setIcon(new ImageIcon(img3));
		stepForwardButton.setMargin(new Insets(0, 0, 0, 0));
		stepForwardButton.setToolTipText("Step one step forward");
		stepForwardButton.setVisible(true);
		
		startButton.setIcon(new ImageIcon(img4));
		startButton.setMargin(new Insets(0, 0, 0, 0));
		startButton.setToolTipText("Start Animation");
		startButton.setVisible(true);
		
		reverseButton.setIcon(new ImageIcon(img5));
		reverseButton.setMargin(new Insets(0, 0, 0, 0));
		reverseButton.setToolTipText("Start Animation in Reverse");
		reverseButton.setVisible(true);
		
		stepBackButton.setIcon(new ImageIcon(img6));
		stepBackButton.setMargin(new Insets(0, 0, 0, 0));
		stepBackButton.setToolTipText("Step one step back");
		stepBackButton.setVisible(true);
		
		stopButton.setEnabled(false);
		stopButton.setIcon(new ImageIcon(img7));
		stopButton.setMargin(new Insets(0, 0, 0, 0));
		stopButton.setToolTipText("Stop Animation");
		stopButton.setVisible(true);
		
		resetParticlesButton.setIcon(new ImageIcon(img8));
		resetParticlesButton.setMargin(new Insets(0, 0, 0, 0));
		resetParticlesButton.setToolTipText("Reset Particles - Does not include exploded ones.");
		resetParticlesButton.setVisible(true);
		
		resetViewButton.setIcon(new ImageIcon(img9));
		resetViewButton.setMargin(new Insets(0, 0, 0, 0));
		resetViewButton.setToolTipText("Reset View");
		resetViewButton.setVisible(true);
		
		mainMenu.setVisible(true);
		
		fileMenu.setText("File");
		fileMenu.setVisible(true);
		
		fileMenuNew.setText("New");
		fileMenuNew.setVisible(true);
		
		fileMenuSeparator2.setVisible(true);
		
		fileMenuSave.setText("Save");
		fileMenuSave.setVisible(true);
		
		fileMenuSaveAs.setText("Save As");
		fileMenuSaveAs.setVisible(true);
		
		fileMenuSeparator3.setVisible(true);
		
		fileMenuOpen.setText("Open");
		fileMenuOpen.setVisible(true);
		
		fileMenuSeparator1.setVisible(true);
		
		fileMenuPrint.setText("Print");
		fileMenuPrint.setVisible(true);
		
		fileMenuSeparator4.setVisible(true);
		
		fileMenuQuit.setText("Quit");
		fileMenuQuit.setVisible(true);
		
		jMenuEdit.setText("Edit");
		jMenuEdit.setVisible(true);
		
		jMenuEditSettings.setText("Settings");
		jMenuEditSettings.setVisible(true);
		
		jMenuTools.setText("Tools");
		jMenuTools.setVisible(true);
		
		jMenuToolsTrackers.setText("Trackers");
		jMenuToolsTrackers.setVisible(true);
		
		jMenuTrackersCreateTimeTracker.setText("Create Time Tracker");
		jMenuTrackersCreateTimeTracker.setVisible(true);
		
		jMenuTrackersCreatePositionTracker.setText("Create Position Tracker");
		jMenuTrackersCreatePositionTracker.setToolTipText("Tracks a Particles Position");
		jMenuTrackersCreatePositionTracker.setVisible(true);
		
		jMenuTrackersCreateVelocityTracker.setText("Create Velocity Tracker");
		jMenuTrackersCreateVelocityTracker.setVisible(true);
		
		jMenuTrackersCreateMomentumTracker.setText("Create Momentum Tracker");
		jMenuTrackersCreateMomentumTracker.setVisible(true);
		
		jMenuTrackersCreateKineticEnergyTracker.setText("Create Kinetic Energy Tracker");
		jMenuTrackersCreateKineticEnergyTracker.setVisible(true);
		
		jMenuTrackersCreateDistanceTracker.setText("Create Distance Tracker");
		jMenuTrackersCreateDistanceTracker.setVisible(true);
		
		jMenuTrackersCreateOrientationTracker.setText("Create Orienation Tracker");
		jMenuTrackersCreateOrientationTracker.setVisible(true);
		
		jMenuTrackersCreateAngularVelocityTracker.setText("Create Angular Velocity Tracker");
		jMenuTrackersCreateAngularVelocityTracker.setVisible(true);
		
		jMenuToolsCreateRandomParticleGrid.setText("Create Random Particle Grid");
		jMenuToolsCreateRandomParticleGrid.setVisible(true);
		
		jMenuHelp.setText("Help");
		jMenuHelp.setVisible(true);
		
		jMenuHelpAbout.setText("About");
		jMenuHelpAbout.setVisible(true);
		
		statusPanel.setLayout(new BorderLayout());
		statusPanel.setVisible(true);
		
		statusLabel.setBorder(new javax.swing.plaf.BorderUIResource.EtchedBorderUIResource(0, Color.white, new Color(142, 142, 142)));
		statusLabel.setLocation(new Point(0, 0));
		statusLabel.setSize(new Dimension(358, 477));
		statusLabel.setText("Ready");
		statusLabel.setVisible(true);
		
		setJMenuBar(mainMenu);
		setIconImage(img0);
		getContentPane().setLayout(new BorderLayout(0, 0));
		setLocation(new Point(0, 0));
		setTitle("Physics3D");

		buttonPanel.add(addParticleButton);
		buttonPanel.add(removeButton);
		buttonPanel.add(stepForwardButton);
		buttonPanel.add(startButton);
		buttonPanel.add(reverseButton);
		buttonPanel.add(stepBackButton);
		buttonPanel.add(stopButton);
		buttonPanel.add(resetParticlesButton);
		buttonPanel.add(resetViewButton);
		
		mainMenu.add(fileMenu);
		mainMenu.add(jMenuEdit);
		mainMenu.add(jMenuTools);
		mainMenu.add(jMenuHelp);
		fileMenu.add(fileMenuNew);
		fileMenu.add(fileMenuSeparator2);
		fileMenu.add(fileMenuSave);
		fileMenu.add(fileMenuSaveAs);
		fileMenu.add(fileMenuSeparator3);
		fileMenu.add(fileMenuOpen);
		fileMenu.add(fileMenuSeparator1);
		fileMenu.add(fileMenuPrint);
		fileMenu.add(fileMenuSeparator4);
		fileMenu.add(fileMenuQuit);
		jMenuEdit.add(jMenuEditSettings);
		jMenuTools.add(jMenuToolsTrackers);
		jMenuTools.add(jMenuToolsCreateRandomParticleGrid);
		jMenuToolsTrackers.add(jMenuTrackersCreateTimeTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreatePositionTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateVelocityTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateMomentumTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateKineticEnergyTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateOrientationTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateAngularVelocityTracker);
		jMenuToolsTrackers.add(jMenuTrackersCreateDistanceTracker);
		jMenuHelp.add(jMenuHelpAbout);
		
		statusPanel.add(statusLabel);
		
		getContentPane().add(main3DPanel, "Center");
		getContentPane().add(buttonPanel, "West");
		getContentPane().add(statusPanel, "South");

		setSize(new java.awt.Dimension(800, 600));

		// event handling
		addParticleButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addParticleButtonActionPerformed(e);
			}
		});
		addParticleButton.addMouseListener(this);
		
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeButtonActionPerformed(e);
			}
		});
		
		removeButton.addMouseListener(this);
		
		stepForwardButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stepForwardButtonActionPerformed(e);
			}
		});
		
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startButtonActionPerformed(e);
			}
		});
		
		startButton.addMouseListener(this);
		
		
		reverseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				reverseButtonActionPerformed(e);
			}
		});
		reverseButton.addMouseListener(this);

		stepBackButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stepBackButtonActionPerformed(e);
			}
		});
		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopButtonActionPerformed(e);
			}
		});
		stopButton.addMouseListener(this);
		
		resetParticlesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetParticlesButtonActionPerformed(e);
			}
		});
		resetParticlesButton.addMouseListener(this);
		
		resetViewButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetViewButtonActionPerformed(e);
			}
		});
		
		resetViewButton.addMouseListener(this);
		
		fileMenuNew.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuNewActionPerformed(e);
			}
		});
		fileMenuSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuSaveActionPerformed(e);
			}
		});
		fileMenuSaveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuSaveAsActionPerformed(e);
			}
		});
		fileMenuOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuOpenActionPerformed(e);
			}
		});
		fileMenuPrint.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuPrintActionPerformed(e);
			}
		});
		fileMenuQuit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fileMenuQuitActionPerformed(e);
			}
		});
		jMenuEditSettings.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jMenuEditSettingsActionPerformed(e);
			}
		});
		jMenuTrackersCreateTimeTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateTimeTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreatePositionTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreatePositionTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreateVelocityTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateVelocityTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreateMomentumTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateMomentumTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreateKineticEnergyTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateKineticEnergyTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreateOrientationTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateOrientationTrackerActionPerformed(e);
			}
		});
		jMenuTrackersCreateAngularVelocityTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateAngularVelocityTrackerActionPerformed(e);
			}
		});
		
		jMenuTrackersCreateDistanceTracker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateDistanceTrackerActionPerformed(e);
			}
		});
		jMenuToolsCreateRandomParticleGrid.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CreateRandomParticleGridActionPerformed(e);
			}
		});
		jMenuHelpAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				HelpAboutActionPerformed(e);
			}
		});
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				thisWindowClosing(e);
			}
		});


	//Give the Properties frame a reference to this frame for positioning purposes
	PropertiesFrame.mainFrame = this;
	
	
		
	//Setup Canvas3D and Simple Universe
	Dimension screenSize = getToolkit().getScreenSize();
	setLocation(((int)screenSize.getWidth() / 2) - (this.getWidth() / 2), ((int)screenSize.getHeight() / 2) - (this.getHeight() / 2));
	c = new Canvas3D(GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices()[0].getBestConfiguration(new GraphicsConfigTemplate3D()));
	offScreenCanvas3D = new OffScreenCanvas3D(SimpleUniverse.getPreferredConfiguration(), true);
	
	// Set the off-screen size based on a scale factor times the
	// on-screen size
	Screen3D sOn = c.getScreen3D();
	Screen3D sOff = offScreenCanvas3D.getScreen3D();
	Dimension dim = sOn.getSize();
	sOff.setSize(dim);
	sOff.setPhysicalScreenWidth(sOn.getPhysicalScreenWidth());
	sOff.setPhysicalScreenHeight(sOn.getPhysicalScreenHeight());
	
	//Create the Universe
	SimpleUniverse u = new SimpleUniverse(c);
	
	//Add the canvas to the main panel
	main3DPanel.add(u.getCanvas());

	// attach the offscreen canvas to the view
    u.getViewer().getView().addCanvas3D(offScreenCanvas3D);
	
	//Create the Main Thread
	t = new Physics3DThread(u, this);
	
	//Add the Thread Orbit Behavior to the Canvas
	c.addMouseListener(t);
	
	//Set the default view
	t.setView(10);
	
	//Start up the Main Thread
	t.start();
	
/////////////////////////////////////

		
		
	}

	// Close the window when the close box is clicked
	void thisWindowClosing(java.awt.event.WindowEvent e) {
		t.stopMe();
		setVisible(false);
		dispose();
		System.exit(0);
	}
	
	public void addParticleButtonActionPerformed(java.awt.event.ActionEvent e) {
		
		t.addParticle();
	
	}
	
	public void startButtonActionPerformed(java.awt.event.ActionEvent e) {
		t.setRewind(false);
		statusLabel.setText("Simulating. Press Stop to continue");
		disable();
		t.setAnimating(true);
	}
	
	public void reverseButtonActionPerformed(java.awt.event.ActionEvent e) {
		t.setRewind(true);
		statusLabel.setText("Simulating. Press Stop to continue");
		disable();
		t.setAnimating(true);
	
	}

	public void stopButtonActionPerformed(java.awt.event.ActionEvent e) {
		statusLabel.setText("Ready");
		enable();
		t.setAnimating(false);
	
	}
	
	public void stepForwardButtonActionPerformed(java.awt.event.ActionEvent e) {
		t.setRewind(false);
		statusLabel.setText("Stepping. Please Wait...");
		disable();
		stopButton.setEnabled(false);
		t.step();
		enable();
	}


	public void stepBackButtonActionPerformed(java.awt.event.ActionEvent e) {
		t.setRewind(true);
		statusLabel.setText("Stepping. Please Wait...");
		disable();
		stopButton.setEnabled(false);
		t.step();
		statusLabel.setText("Ready");
		enable();
		
	}
	
	public void enable()
	{
		statusLabel.setText("Ready");
		startButton.setEnabled(true);
		stepForwardButton.setEnabled(true);
		stepBackButton.setEnabled(true);
		reverseButton.setEnabled(true);
		stopButton.setEnabled(false);
		addParticleButton.setEnabled(true);
		removeButton.setEnabled(true);
		resetParticlesButton.setEnabled(true);
		fileMenu.setEnabled(true);
		t.setAnimating(false);
	}
	
	public void disable()
	{
		statusLabel.setText("Simulating. Press Stop to continue");
		startButton.setEnabled(false);
		stepForwardButton.setEnabled(false);
		stepBackButton.setEnabled(false);
		reverseButton.setEnabled(false);
		stopButton.setEnabled(true);
		addParticleButton.setEnabled(false);
		removeButton.setEnabled(false);
		resetParticlesButton.setEnabled(false);
		fileMenu.setEnabled(false);
	}
	
	public void resetViewButtonActionPerformed(java.awt.event.ActionEvent e) {
		double zoomDistance;
		try
		{
			String result = (String)JOptionPane.showInputDialog("Zoom Distance?", "10");
			if(result == null)
				return;
				
			zoomDistance =  Double.parseDouble(result);
			t.setView(zoomDistance);
		}
		catch(Exception exp)
		{
			
			JOptionPane.showMessageDialog(this, "Error: You did not input a number. Please Try Again.", 
											"Error!",  JOptionPane.ERROR_MESSAGE);
		}
		
	}
	
	
	public void resetParticlesButtonActionPerformed(java.awt.event.ActionEvent e) {
		
		Particle3D.resetAll();
		t.pframe.reset();
		
	}
	
	
	public void fileMenuQuitActionPerformed(java.awt.event.ActionEvent e) {
	
		t.stopMe();
		setVisible(false);
		dispose();
		System.exit(0);
		
	}
	
	public void jMenuEditSettingsActionPerformed(java.awt.event.ActionEvent e) {
		try
		{
			SettingsFrame sframe = new SettingsFrame(this);
			sframe.initComponents();
			sframe.setVisible(true);
		}
		catch(Exception exp)
		{
		
		}
	
	}
	
	public void removeButtonActionPerformed(java.awt.event.ActionEvent e) {
		t.removeSelectedObject();
	}
	
	
	
	public void fileMenuSaveActionPerformed(java.awt.event.ActionEvent e) {
		
		try
		{
			if(fileName != null)
			{
				BufferedWriter b = new BufferedWriter(new FileWriter(fileName));	
				Particle3D.saveAll(b);
				b.close();
			}
			else
			{
				chooser = new JFileChooser();
				
				ExampleFileFilter filter = new ExampleFileFilter();
   				filter.addExtension("p3d");
    			filter.setDescription("Physics 3D Files");
   				chooser.setFileFilter(filter);
				
				//Get the new file
				int fileChooserReturn = chooser.showSaveDialog(this);
				//If the user said ok save the colony to the selected file
				if(fileChooserReturn == JFileChooser.APPROVE_OPTION)
				{
					fileName = chooser.getSelectedFile();
					if(!fileName.getName().endsWith(".p3d"))
						fileName = new File(fileName.getAbsolutePath() + ".p3d");
						
					BufferedWriter b = new BufferedWriter(new FileWriter(fileName));	
					Particle3D.saveAll(b);
					b.close();
					this.setTitle("Physics 3D - " + fileName.getName());
				}
			}
				
		}
		//If there is an error writing to the file, show an error dialog
		catch(IOException exc)
		{
			JOptionPane.showMessageDialog(this, "Error opening file for output", "Error!",  JOptionPane.ERROR_MESSAGE);
		}
	
	}
		
	
	
	public void fileMenuOpenActionPerformed(java.awt.event.ActionEvent e) {
						
			try
			{
				chooser = new JFileChooser("Examples");
				
				ExampleFileFilter filter = new ExampleFileFilter();
   				filter.addExtension("p3d");
    			filter.setDescription("Physics 3D Files");
   				chooser.setFileFilter(filter);
				
				//Get the new file
				int fileChooserReturn = chooser.showDialog(this, "Load");
				//If the user said ok load the new file
				if(fileChooserReturn == JFileChooser.APPROVE_OPTION)
				{	
					fileName = chooser.getSelectedFile();
					BufferedReader r = new BufferedReader(new FileReader(chooser.getSelectedFile()));
					Particle3D.removeAll();
					Particle3D.loadAll(r, t);
					r.close();
					this.setTitle("Physics 3D - " + fileName.getName());
				}
				
			}
			//If the file wasn't found, show an error dialog
			catch(FileNotFoundException exc)
			{
				JOptionPane.showMessageDialog(this, "File not Found", "Error!",  JOptionPane.ERROR_MESSAGE);
			}
			catch(IOException exc)
			{
				JOptionPane.showMessageDialog(this, "Corrupt Save File", "Error!",  JOptionPane.ERROR_MESSAGE);
			}
			catch(NumberFormatException exc)
			{
				JOptionPane.showMessageDialog(this, "Corrupt Save File", "Error!",  JOptionPane.ERROR_MESSAGE);

			}
	
	}
	
	public void fileMenuNewActionPerformed(java.awt.event.ActionEvent e) {
		fileName = null;
		this.setTitle("Physics3D");
		Particle3D.removeAll();
	}
	
	public void fileMenuSaveAsActionPerformed(java.awt.event.ActionEvent e) {
		try
		{
			chooser = new JFileChooser();
			//Get the new file
			int fileChooserReturn = chooser.showSaveDialog(this);
			//If the user said ok save the colony to the selected file
			if(fileChooserReturn == JFileChooser.APPROVE_OPTION)
			{
				fileName = chooser.getSelectedFile();
				BufferedWriter b = new BufferedWriter(new FileWriter(chooser.getSelectedFile()));	
				Particle3D.saveAll(b);
				b.close();
				this.setTitle("Physics 3D - " + fileName.getName());
			}
				
		}
		//If there is an error writing to the file, show an error dialog
		catch(IOException exc)
		{
			JOptionPane.showMessageDialog(this, "Error opening file for output", "Error!",  JOptionPane.ERROR_MESSAGE);
		}
	
	}
	
	public void CreatePositionTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Position Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		PositionParticle3DTracker tracker = new PositionParticle3DTracker(p);
	}
	
	public void CreateVelocityTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Velocity Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		VelocityParticle3DTracker tracker = new VelocityParticle3DTracker(p);

	}
	
	public void CreateOrientationTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Position Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		OrientationParticle3DTracker tracker = new OrientationParticle3DTracker(p);
	}
	
	public void CreateAngularVelocityTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Velocity Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		AngularVelocityParticle3DTracker tracker = new AngularVelocityParticle3DTracker(p);

	}
	
	public void CreateDistanceTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p1 = (Particle3D)JOptionPane.showInputDialog(this, "Select the First Particle to Track", "Create Distance Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p1 == null)
			return;
			
		Particle3D p2 = (Particle3D)JOptionPane.showInputDialog(this, "Select the Second Particle to Track", "Create Distance Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p2 == null)
			return;
			
		DistanceParticle3DTracker tracker = new DistanceParticle3DTracker(p1, p2);
	
	}
	
	public void CreateMomentumTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Momentum Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		MomentumParticle3DTracker tracker = new MomentumParticle3DTracker(p);

	
	}
	
	
	public void CreateKineticEnergyTrackerActionPerformed(java.awt.event.ActionEvent e) {
		Object[] particles = Particle3D.getParticles();
		if(particles.length <= 0)
		{
			JOptionPane.showMessageDialog(this, "Error: You must create an object in order to track it", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Particle3D p = (Particle3D)JOptionPane.showInputDialog(this, "Select the Particle to Track", "Create Kinetic Energy Tracker",
									            JOptionPane.INFORMATION_MESSAGE, null, particles, 
									            particles[0]);
		if(p == null)
			return;
			
		KineticEnergyParticle3DTracker tracker = new KineticEnergyParticle3DTracker(p);

	
	}
	
	

	
	public void CreateTimeTrackerActionPerformed(java.awt.event.ActionEvent e) {
		
		TimeParticle3DTracker tracker = new TimeParticle3DTracker();
		
	}
		
	
	//Printing....
	
	
	
	public int print(Graphics g, PageFormat pf, int pageNumber)
	{
		Graphics2D g2d = (Graphics2D)g;
		
		Dimension dPage = new Dimension((int)pf.getImageableWidth(), (int)pf.getImageableHeight());	
		g2d.translate((int)pf.getImageableX(),(int)pf.getImageableY());
		
	
		
		if(pageNumber == 0)
		{
			g2d.drawImage(offScreenCanvas3D.doRender(2*(int)dPage.getWidth(), 2*(int)dPage.getHeight()), 0,0, (int)dPage.getWidth(), (int)dPage.getHeight(), null);
				
		}
		else if(pageNumber == 1)
		{
			
			if(!Particle3DTracker.printAllTrackers(g2d, dPage))
				return NO_SUCH_PAGE;
		}
		else
			return(NO_SUCH_PAGE);
		
		
		return PAGE_EXISTS;
	}
	
	public void fileMenuPrintActionPerformed(java.awt.event.ActionEvent e) {
		
		PrinterJob pj = PrinterJob.getPrinterJob();
		
		pj.setPrintable(this);
		
		if(pj.printDialog())
		{
			try
			{
				pj.print();
			}
			catch(PrinterException exp)
			{
				exp.printStackTrace();
			}

		}
	}
	
	//////////////////////////////
		
	//Creates a x by x grid of particles with random velocities between -1 and 1
	public void CreateRandomParticleGridActionPerformed(java.awt.event.ActionEvent e) {
	
		int magnitude = 2;
		int size = -1;
		
		try
		{
			String sSize = JOptionPane.showInputDialog(this, "Size of Grid?", "3");
			if(sSize == null)
				return;
			
			size = Integer.parseInt(sSize);
		}
		catch(NumberFormatException exp)
		{
			JOptionPane.showMessageDialog(this, "Error You did not enter a number", "Error", JOptionPane.ERROR_MESSAGE);
		}
		
		this.fileMenuNewActionPerformed(e);
		
		int numFromCenterLeft = size / 2;
		int numFromCenterRight = numFromCenterLeft;
		
		if(size % 2 == 0)
			numFromCenterRight--; 
				
		for(int x = 0 - numFromCenterLeft; x <= 0 + numFromCenterRight; x++)
			for(int y = 0 - numFromCenterLeft; y <= 0 + numFromCenterRight; y++)
				for(int z = 0 - numFromCenterLeft; z <= 0 + numFromCenterRight; z++)
				{
					Vector3d velocity = new Vector3d(magnitude * (Math.random() - .5), magnitude * (Math.random() - .5), magnitude * (Math.random() - .5));
					t.addParticle("P" + x + y + z, new Vector3d(x, y, z), velocity, 100, .1, 1);
				}
		
	}	
	/////////////////////////////	
																											
	
	public void HelpAboutActionPerformed(java.awt.event.ActionEvent e) {
		try
		{
			AboutFrame f = new AboutFrame();
			f.initComponents();
			f.setVisible(true);	
		}
		catch(Exception exp)
		{
			exp.printStackTrace();
		}
	}
	
	
	//////////////////
	
	//Mouse Events /////////////////////////////////
	public void mouseEntered(MouseEvent e)
	{
		if(e.getComponent() instanceof JButton && e.getComponent().isEnabled())
			statusLabel.setText(((JButton)e.getComponent()).getToolTipText());
	}
	
	public void mouseExited(MouseEvent e)
	{
		if(startButton.isEnabled())
			statusLabel.setText("Ready");
		else
			statusLabel.setText("Simulating. Press Stop to continue");
	}
	
	public void mouseClicked(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e){}
	////////////////////////////////////////
												
}
	
		
	
	